@@ -24,18 +24,18 @@
 
 #include <config.h>
 
-#include "PrinterModel.h"
+#include <PrinterModel.h>
+#include <PrinterSortFilterModel.h>
 #include "PrinterDelegate.h"
 #include "PrinterDescription.h"
 
 #include <KMessageBox>
 #include <KGenericFactory>
 #include <KAboutData>
+#include <KToolInvocation>
 #include <KIcon>
 
 #include <QMenu>
-#include <QDBusMessage>
-#include <QDBusConnection>
 #include <KCupsRequest.h>
 #include <KCupsServer.h>
 #include <NoSelectionRectDelegate.h>
@@ -43,7 +43,7 @@
 #include <cups/cups.h>
 
 K_PLUGIN_FACTORY(PrintKCMFactory, registerPlugin<PrintKCM>();)
-K_EXPORT_PLUGIN(PrintKCMFactory("kcm_print"))
+K_EXPORT_PLUGIN(PrintKCMFactory("kcm_print", "print-manager"))
 
 PrintKCM::PrintKCM(QWidget *parent, const QVariantList &args) :
     KCModule(PrintKCMFactory::componentData(), parent, args),
@@ -58,15 +58,18 @@ PrintKCM::PrintKCM(QWidget *parent, const QVariantList &args) :
                                PM_VERSION,
                                ki18n("Print settings"),
                                KAboutData::License_GPL,
-                               ki18n("(C) 2010-2012 Daniel Nicoletti"));
+                               ki18n("(C) 2010-2013 Daniel Nicoletti"));
     setAboutData(aboutData);
     setButtons(NoAdditionalButton);
-    KGlobal::insertCatalog(QLatin1String("print-manager"));
 
     ui->setupUi(this);
     
     connect(ui->printerDesc, SIGNAL(updateNeeded()), SLOT(update()));
 
+    // The printer list needs to increase in width according to the icon sizes
+    // default dialog icon size is 32, this times 6 is 192 which is roughly the original width
+    ui->printersTV->setMinimumWidth(IconSize(KIconLoader::Dialog) * 6);
+
     QMenu *addMenu = new QMenu(this);
     addMenu->addAction(i18nc("@action:intoolbar","Add a Printer Class"),
                        this, SLOT(addClass()));
@@ -102,19 +105,21 @@ PrintKCM::PrintKCM(QWidget *parent, const QVariantList &args) :
     ui->systemPreferencesTB->setToolTip(i18n("Configure the global preferences"));
     ui->systemPreferencesTB->setMenu(systemMenu);
 
-    m_model = new PrinterModel(winId(), this);
-    ui->printersTV->setModel(m_model);
+    m_model = new PrinterModel(this);
+    PrinterSortFilterModel *sortModel = new PrinterSortFilterModel(this);
+    sortModel->setSourceModel(m_model);
+    ui->printersTV->setModel(sortModel);
     ui->printersTV->setItemDelegate(new NoSelectionRectDelegate(this));
     ui->printersTV->setItemDelegate(new PrinterDelegate(this));
     connect(ui->printersTV->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
             this, SLOT(update()));
-    connect(m_model, SIGNAL(rowsInserted(QModelIndex,int,int)),
+    connect(sortModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
             this, SLOT(update()));
-    connect(m_model, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+    connect(sortModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
             this, SLOT(update()));
-    connect(ui->printersTV->model(), SIGNAL(dataChanged(QModelIndex,QModelIndex)),
+    connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
             this, SLOT(update()));
-    connect(ui->printersTV->model(), SIGNAL(error(int,QString,QString)),
+    connect(m_model, SIGNAL(error(int,QString,QString)),
             this, SLOT(error(int,QString,QString)));
 
     ui->addPrinterBtn->setIcon(KIcon("list-add"));
@@ -199,6 +204,7 @@ void PrintKCM::showInfo(const KIcon &icon, const QString &title, const QString &
 void PrintKCM::update()
 {
     if (m_model->rowCount()) {
+        m_lastError = -1; // if the model has printers reset the error code
         if (ui->stackedWidget->currentIndex() != 0) {
             ui->stackedWidget->setCurrentIndex(0);
         }
@@ -223,12 +229,9 @@ void PrintKCM::update()
         }
         ui->printerDesc->setDestName(index.data(PrinterModel::DestName).toString(),
                                      index.data(PrinterModel::DestDescription).toString(),
-                                     index.data(PrinterModel::DestIsClass).toBool());
-        if (m_model->rowCount() == 1) {
-          ui->printerDesc->setDestStatus(index.data(PrinterModel::DestStatus).toString());
-        } else {
-          ui->printerDesc->setDestStatus(QString());
-        }
+                                     index.data(PrinterModel::DestIsClass).toBool(),
+                                     m_model->rowCount() == 1);
+        ui->printerDesc->setDestStatus(index.data(PrinterModel::DestStatus).toString());
         ui->printerDesc->setLocation(index.data(PrinterModel::DestLocation).toString());
         ui->printerDesc->setKind(index.data(PrinterModel::DestKind).toString());
         ui->printerDesc->setIsShared(index.data(PrinterModel::DestIsShared).toBool());
@@ -259,24 +262,16 @@ void PrintKCM::update()
 
 void PrintKCM::on_addTB_clicked()
 {
-    QDBusMessage message;
-    message = QDBusMessage::createMethodCall(QLatin1String("org.kde.AddPrinter"),
-                                             QLatin1String("/"),
-                                             QLatin1String("org.kde.AddPrinter"),
-                                             QLatin1String("AddPrinter"));
-    message << static_cast<qulonglong>(winId());
-    QDBusConnection::sessionBus().call(message);
+    QStringList args;
+    args << "--add-printer";
+    KToolInvocation::kdeinitExec(QLatin1String("kde-add-printer"), args);
 }
 
 void PrintKCM::addClass()
 {
-    QDBusMessage message;
-    message = QDBusMessage::createMethodCall(QLatin1String("org.kde.AddPrinter"),
-                                             QLatin1String("/"),
-                                             QLatin1String("org.kde.AddPrinter"),
-                                             QLatin1String("AddClass"));
-    message << static_cast<qulonglong>(winId());
-    QDBusConnection::sessionBus().call(message);
+    QStringList args;
+    args << "--add-class";
+    KToolInvocation::kdeinitExec(QLatin1String("kde-add-printer"), args);
 }
 
 void PrintKCM::on_removeTB_clicked()
@@ -313,7 +308,9 @@ void PrintKCM::on_removeTB_clicked()
 void PrintKCM::getServerSettings()
 {
     if (!m_serverRequest) {
+        QMenu *systemMenu = qobject_cast<QMenu*>(sender());
         m_serverRequest = new KCupsRequest;
+        m_serverRequest->setProperty("interactive", static_cast<bool>(systemMenu));
         connect(m_serverRequest, SIGNAL(finished()),
                 this, SLOT(getServerSettingsFinished()));
         m_serverRequest->getServerSettings();
@@ -336,10 +333,12 @@ void PrintKCM::getServerSettingsFinished()
     m_allowUsersCancelAnyJob->setEnabled(!error);
 
     if (error) {
-        KMessageBox::detailedSorry(this,
-                                   i18nc("@info", "Failed to get server settings"),
-                                   request->errorMsg(),
-                                   i18nc("@title:window", "Failed"));
+        if (request->property("interactive").toBool()) {
+            KMessageBox::detailedSorry(this,
+                                       i18nc("@info", "Failed to get server settings"),
+                                       request->errorMsg(),
+                                       i18nc("@title:window", "Failed"));
+        }
     } else {
         KCupsServer server = request->serverSettings();
 
